#!/usr/bin/perl -w
#
# sarlog.pl
#
# transform the output of sar -abcgkmpqruvwy 
# into a format easily read by cricket
#
# Copyright (c) Brook Schofield <B.Schofield@mailbox.gu.edu.au> 2002
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
use strict;

unless (@ARGV) {
	die("Usage: $0 <sar_file>\n");
}

#
# Determine the order that the sar output will be shown
#
my @OUTPUTORDER = qw(iget/s namei/s dirbk/s bread/s lread/s %rcache bwrit/s lwrit/s %wcache pread/s pwrit/s scall/s sread/s swrit/s fork/s exec/s rchar/s wchar/s pgout/s ppgout/s pgfree/s pgscan/s %ufs_ipf sml_mem alloc fail lg_mem lg_memalloc lg_memfail ovsz_alloc ovsz_allocfail msg/s sema/s atch/s pgin/s ppgin/s pflt/s vflt/s slock/s runq-sz %runocc swpq-sz %swpocc freemem freeswap %usr %sys %wio %idle proc-sz proc-szmax ov inod-sz inod-szmax inod-szov file-sz file-szmax file-szov lock-sz lock-szmax swpin/s bswin/s swpot/s bswot/s pswch/s rawch/s canch/s outch/s rcvin/s xmtin/s mdmin/s);

my %stats;
my($header,$line);
my $FILE = shift;
open(INPUT, $FILE) or die("$! \"$FILE\"\n");

while(<INPUT>) {
	if (/^\d+:\d+\d+/) {
		$header = $_ unless $line;
		$line = $_;			# Store the lastest sar line
	} else {
		if ($line) {			# After the last time based
						# line is processed
			my ($prevhead);
			chomp($header);
			chomp($line);
			my @header = split(/ +/,$header);
			my @line   = split(/ +/,$line);
			for(my $i = 1; $i <= $#header; $i++) {
				if (defined($line[$i])) {
					if (defined($stats{$header[$i]})) {
						$stats{$prevhead . $header[$i]} = $line[$i];
					} else {
						if ($line[$i] =~ /\//) {
							my ($numerator, $denominator) = split('/',$line[$i]);
							$stats{$header[$i]} = $numerator;
							$stats{$header[$i] . "max"} = $denominator;
						} else {
							$stats{$header[$i]} = $line[$i];
						}
					}
				}
				$prevhead = $header[$i] unless $header[$i] =~ /^(alloc|fail|ov)$/;
			}
		undef $line;
		}
	}
}

foreach my $statname (@OUTPUTORDER) {
	if (defined($stats{$statname})) {
		print $stats{$statname};
	} else {
		#print "U";
		print "0";
	}
	print "\t$statname\n";
}
